/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.json;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.*;
import com.fasterxml.jackson.databind.*;
import com.fasterxml.jackson.databind.deser.ValueInstantiator;
import com.fasterxml.jackson.databind.module.SimpleModule;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import java.io.IOException;
import java.lang.reflect.Type;
import javax.annotation.concurrent.Immutable;
import javax.security.auth.x500.X500Principal;
import net.java.truelicense.core.*;
import net.java.truelicense.core.auth.*;
import net.java.truelicense.json.codec.JsonCodec;
import net.java.truelicense.obfuscate.Obfuscate;

/**
 * The root context for the management of V2/JSON format license keys.
 * Note that there is no interoperability between different format license keys.
 * <p>
 * Use this context to create a {@link LicenseVendorContext} or a
 * {@link LicenseConsumerContext}.
 * Here's an example for verifying the installed license key in a consumer
 * application:
 * <pre><code>
 * LicenseManagementContext mc = new V2JsonLicenseManagementContext("MyApp 1");
 * LicenseConsumerContext cc = mc.consumer();
 * LicenseConsumerManager cm = cc.manager(...);
 * cm.verify();
 * </code></pre>
 * <p>
 * Or in a fluent API programming style:
 * <pre><code>
 * new V2JsonLicenseManagementContext("MyApp 1").consumer().manager()...verify();
 * </code></pre>
 * <p>
 * Do not copy-paste this code!
 * Instead, use the TrueLicense Maven Archetype to generate a fully populated
 * template project for you.
 * <p>
 * Where required, this class should get subclassed to customize license
 * {@linkplain #license creation},
 * {@linkplain #initialization initialization}
 * or {@linkplain #validation validation},
 * {@linkplain #codec encoding} or to implement an authoritative
 * {@linkplain #clock clock} or to provide an alternative
 * {@linkplain #classLoader class loader} for loading key stores.
 * <p>
 * Note that this class is immutable.
 * Unless stated otherwise, all no-argument methods need to return consistent
 * objects so that caching them is not required.
 * A returned object is considered to be consistent if it compares
 * {@linkplain Object#equals(Object) equal} or at least behaves identical to
 * any previously returned object.
 *
 * @author Christian Schlichtherle
 */
@Immutable
public class V2JsonLicenseManagementContext
extends BasicV2LicenseManagementContext {

    @Obfuscate private static final String ARTIFACT_BODY = "artifactBody";
    @Obfuscate private static final String SIGNATURE_BODY = "signatureBody";

    private volatile ObjectMapper mapper;

    /**
     * Constructs a V2/JSON license management context.
     * The provided string should get computed on demand from an obfuscated
     * form, e.g. by annotating a constant string value with the
     * {@literal @}{@link Obfuscate} annotation and processing it with the
     * TrueLicense Maven Plugin.
     *
     * @param subject the licensing subject, i.e. a product name with an
     *        optional version range, e.g. {@code MyApp 1}.
     */
    public V2JsonLicenseManagementContext(String subject) { super(subject); }

    /**
     * {@inheritDoc}
     * <p>
     * The implementation in the class {@link V2JsonLicenseManagementContext}
     * returns a new {@link GenericRepository}.
     */
    @Override public final GenericRepository repository() {
        return new GenericRepository(codec(License.class));
    }

    /**
     * {@inheritDoc}
     * <p>
     * The implementation in the class {@link V2JsonLicenseManagementContext}
     * returns a new {@link JsonCodec}.
     */
    @Override public JsonCodec codec(Type expected) {
        return new JsonCodec(mapper(), expected);
    }

    private ObjectMapper mapper() {
        final ObjectMapper m = mapper;
        return null != m ? m : (mapper = newMapper());
    }

    /**
     * Returns a new object mapper for use with {@linkplain #license licenses}
     * and {@linkplain #repository repositories}.
     * This method is normally only called once.
     * In a multi-threaded environment, it may get called more than once, but
     * then each invocation must return an object which behaves equivalent to
     * any previously returned object.
     */
    protected ObjectMapper newMapper() {
        final ObjectMapper mapper = new ObjectMapper();
        mapper.setSerializationInclusion(JsonInclude.Include.NON_DEFAULT);
        final SimpleModule module = new SimpleModule();

        // A custom serializer is required because X500Principal does
        // not have a default constructor.
        module.addSerializer(new StdSerializer<X500Principal>(X500Principal.class) {
            @Override public void serialize(
                    X500Principal value,
                    JsonGenerator jgen,
                    SerializerProvider provider)
            throws IOException {
                jgen.writeString(value.getName());
            }
        });

        module.addAbstractTypeMapping(Repository.class, GenericRepository.class);

        // A custom serializer is required because GenericRepository does
        // not have a default constructor AND the serialization inclusion is
        // set to JsonInclude.Include.NON_DEFAULT.
        module.addSerializer(new StdSerializer<GenericRepository>(GenericRepository.class) {
            @SuppressWarnings("unchecked")
            @Override public void serialize(
                    GenericRepository value,
                    JsonGenerator jgen,
                    SerializerProvider provider)
            throws IOException {
                jgen.writeStartObject();
                jgen.writeStringField(ARTIFACT_BODY, value.getArtifactBody());
                jgen.writeStringField(SIGNATURE_BODY, value.getSignatureBody());
                jgen.writeEndObject();
            }
        });

        // A custom value instantiator is required because GenericRepository
        // does not have a default constructor.
        module.addValueInstantiator(GenericRepository.class, new ValueInstantiator() {
            @Override public String getValueTypeDesc() {
                return GenericRepository.class.getName();
            }

            @Override public boolean canInstantiate() { return true; }

            @Override public boolean canCreateUsingDefault() { return true; }

            @Override
            public Object createUsingDefault(DeserializationContext ctxt)
            throws IOException {
                return repository();
            }
        });

        mapper.registerModule(module);
        return mapper;
    }
}
